#ifdef	ONE_WIRE

#include <avr/io.h>
#include <util/delay.h>
#include <string.h>

#include "ds18x20.h"
#include "onewire.h"

// find DS18X20 Sensors on 1-Wire-Bus
//   input/ouput: diff is the result of the last rom-search
//   output: id is the rom-code of the sensor found

void DS18X20_find_sensor(uint8_t *diff, uint8_t id[])
{
	for (;;) {
		*diff = ow_rom_search( *diff, &id[0] );
		if ( *diff==OW_PRESENCE_ERR || *diff==OW_DATA_ERR ||
		  *diff == OW_LAST_DEVICE ) return;
		if ( id[0] == DS18B20_ID || id[0] == DS18S20_ID ) return;
	}
}

// start measurement (CONVERT_T) for all sensors if input id==NULL 
//   or for single sensor. then id is the rom-code

uint8_t DS18X20_start_meas(uint8_t id[])
{
	ow_reset(); //**
	if( ow_input_pin_state() ) { // only send if bus is "idle" = high
		ow_command( DS18X20_CONVERT_T, id );

		_delay_ms(10);
		ow_parasite_disable();
		return DS18X20_OK;
	} else
	    return DS18X20_START_FAIL;
}

// reads temperature (scratchpad) of sensor with rom-code id
//   output: subzero==1 if temp.<0, cel: full celsius, mcel: frac 
//   in millicelsius*0.1
//   i.e.: subzero=1, cel=18, millicel=5000 = -18,5000C

uint8_t DS18X20_read_meas(uint8_t id[], uint8_t *subzero, 
	uint8_t *cel, uint8_t *cel_frac_bits)
{
	uint8_t i;
	uint8_t sp[DS18X20_SP_SIZE];
	
	ow_reset(); //**
	ow_command(DS18X20_READ, id);
	for ( i=0 ; i< DS18X20_SP_SIZE; i++ ) sp[i]=ow_byte_rd();

	if ( crc8( &sp[0], DS18X20_SP_SIZE , 0x18 ) ) 
	    return DS18X20_ERROR_CRC;

	uint16_t meas;

	if( id[0] == DS18S20_ID ) { // 9 -> 12 bit if 18S20
		if ( sp[1] > 0 )
		{
			*subzero = 1;
			meas = 0x0100 - sp[0];
		} else {
			*subzero = 0;
			meas = sp[0];
		}
		meas &= 0xfe;
		meas <<= 3;			// Convert to 12-bit , now degrees are in 1/16 degrees units
		meas += (16 - sp[6]); // - 4 ;	// Add the compensation , and remember to subtract 0.25 degree (4/16)

		if (( meas >= 0 ) && ( meas < 4 ))
		{
			*subzero = 1;
			*cel = 0;
			*cel_frac_bits = 4 - meas;
		} else {
			meas -= 4;
			*cel		= (uint8_t) (meas >> 4);
			*cel_frac_bits	= (uint8_t) (meas & 0x000F);
		}
	}
	

	if ((*cel == 85) && (*cel_frac_bits == 0) && ( !*subzero )) return DS18X20_ERROR_CRC;
	return DS18X20_OK;
}

uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];

int cmp_sensor_romcode(const void *first, const void *second)
{
	return  strcmp(first,second);
}


uint8_t DS18X20_search_sensors(void)
{
	uint8_t i;
	uint8_t id[OW_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	nSensors = 0;
	
	for( diff = OW_SEARCH_FIRST; 
		diff != OW_LAST_DEVICE && nSensors < MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		if(( diff == OW_DATA_ERR )||( diff == OW_PRESENCE_ERR ))
		    break;

		for (i=0;i<OW_ROMCODE_SIZE;i++)
			gSensorIDs[nSensors][i]=id[i];
		
		nSensors++;
	}

	qsort(gSensorIDs,nSensors,OW_ROMCODE_SIZE,cmp_sensor_romcode);

	return nSensors;
}

#define CRC8INIT	0x00
#define CRC8POLY	0x18              //0X18 = X^8+X^5+X^4+X^0

uint8_t	crc8 ( uint8_t *data_in, uint16_t number_of_bytes_to_read,uint8_t poly )
{
	uint8_t	 crc;
	uint16_t loop_count;
	uint8_t  bit_counter;
	uint8_t  data;
	uint8_t  feedback_bit;
	
	crc = CRC8INIT;

	for (loop_count = 0; loop_count != number_of_bytes_to_read; loop_count++)
	{
		data = data_in[loop_count];
		
		bit_counter = 8;
		do {
			feedback_bit = (crc ^ data) & 0x01;
	
			if ( feedback_bit == 0x01 ) {
				crc = crc ^ poly;
			}
			crc = (crc >> 1) & 0x7F;
			if ( feedback_bit == 0x01 ) {
				crc = crc | 0x80;
			}
		
			data = data >> 1;
			bit_counter--;
		
		} while (bit_counter > 0);
	}
	
	return crc;
}

// termometer_error == 1 -  
void startDS18X20temperature( uint8_t i )
{
	termometer_error = 0;
	if( DS18X20_start_meas( gSensorIDs[i] ) != DS18X20_OK)
		termometer_error = 1;
}

//  0 =   
// 1 -  
// termometer_error == 1 -  	    
char readDS18X20temperature( uint8_t i, int16_t* Tout )
{
	termometer_error = 0;
	if ( DS18X20_get_state() != 0xff )
		return 0;
	
	uint8_t	subzero, cel, cel_frac_bits;
	if( DS18X20_read_meas(gSensorIDs[i], &subzero, &cel, &cel_frac_bits) != DS18X20_OK )
	{
		termometer_error = 1;	
		return 1;
	}

	int16_t	result;

	result = cel * 100;
	result += (cel_frac_bits * DS18X20_FRACCONV)/100;

	/*
	1	0625		0.06
	2	1250	2	0.13
	3	1875	3	0.19
	4	2500		0.25
	5	3125		0.31
	6	3750	6	0.38
	7	4375	7	0.44
	8	5000		0.50
	9	5625		0.56
	10	6250	10	0.63
	11	6875	11	0.69
	12	7500		0.75
	13	8125		0.81
	14	8750	14	0.88
	15	9375	15	0.94
	*/
	uint8_t need_rounding[] = { 2, 3, 6, 7, 10, 11, 14, 15 };
	uint8_t k = 0;
	while ( k < sizeof( need_rounding ) ) {
		if ( cel_frac_bits == need_rounding[ k ] ) {
			result++;
			break;
		}
		k++;
	}

	if( subzero ) result = -result;

	*Tout = result;

	return 1;
} 	  	

#endif
